//#include <stdint.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/i2c.h>
#include <linux/i2c-proc.h>
#include <linux/init.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/fs.h>
#include <linux/delay.h>
#include <asm/uaccess.h>
#include <video/tv_tuner.h>
#include "hxt5g02a.h"

#define		LM_DATE		"20060906"
#define		LM_VERSION	"1.0.0.0"
MODULE_LICENSE("GPL");

// Addresses to scan 
static unsigned short normal_i2c[] = { HXT5G02A_I2C_IOADDR, HXTTUNER_I2C_IOADDR, SENSORS_I2C_END };
static unsigned short normal_i2c_range[] = { SENSORS_I2C_END };
static unsigned int normal_isa[] = { SENSORS_ISA_END };
static unsigned int normal_isa_range[] = { SENSORS_ISA_END };

// Insmod parameters 
SENSORS_INSMOD_2(HXT5G02A, HXTTUNER);

//	tuner system
static int HXT5G02AAttachAdapter(struct i2c_adapter *adapter);
static int HXT5G02ADetect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind);
static int HXT5G02ADetachClient(struct i2c_client *client);
static void HXT5G02AProcRead(struct i2c_client *client, int operation, int ctl_name, int *nrels_mag, long *results);
static void HXT5G02AProcWrite(struct i2c_client *client, int operation, int ctl_name, int *nrels_mag, long *results);
static void HXT5G02AUpdateClient(struct i2c_client *client);
static void HXT5G02AInitClient(struct i2c_client *client);
//	tuner

static int HXTTunerAttachAdapter(struct i2c_adapter *adapter);
static int HXTTunerDetect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind);
static int HXTTunerDetachClient(struct i2c_client *client);
static void HXTTunerProcRead(struct i2c_client *client, int operation, int ctl_name, int *nrels_mag, long *results);
static void HXTTunerProcWrite(struct i2c_client *client, int operation, int ctl_name, int *nrels_mag, long *results);
static void HXTTunerUpdateClient(struct i2c_client *client);
static void HXTTunerInitClient(struct i2c_client *client);

//	Tuner system for internal setup
static int HXT5G02AOpen(struct inode *pinode, struct file *pfile);
static int HXT5G02ARelese(struct inode *pinode, struct file *pfile);
static int HXT5G02AIOCTL(struct inode *pinode, struct file *pfile, 
	unsigned int cmd, unsigned long arg);
//	Tuner for RF tunning
static int HXTTunerOpen(struct inode *pinode, struct file *pfile);
static int HXTTunerRelese(struct inode *pinode, struct file *pfile);
static int HXTTunerIOCTL(struct inode *pinode, struct file *pfile, 
	unsigned int cmd, unsigned long arg);

static const char HXT5G02A_SHORT_NAME[]="HXT5G02A";
static const char HXT5G02A_FULL_NAME[]="Himax HXT5G02A TV tuner system";
static const char HXT5G02A_VERSION[]="1.0.0.0";

static const char HXTTUNER_SHORT_NAME[]="HXTTUNER";
static const char HXTTUNER_FULL_NAME[]="Himax HXTUNER TV tuner";
static const char HXTTUNER_VERSION[]="1.0.0.0";

static struct file_operations HXT5G02AFOPS=
{
	owner:		THIS_MODULE,
	open:		HXT5G02AOpen,
	release:	HXT5G02ARelese,
	ioctl:		HXT5G02AIOCTL,
};

static struct file_operations HXTTUNERFOPS=
{
	owner:		THIS_MODULE,
	open:		HXTTunerOpen,
	release:	HXTTunerRelese,
	ioctl:		HXTTunerIOCTL,
};

TUNERSYSTEMDATA g_SystemData[6]=
{
	{
		{
			HXT5G02A_SYS_NTSCMN_SWM, 
			HXT5G02A_SYS_NTSCMN_ADJM, 
			HXT5G02A_SYS_NTSCMN_DATM
		}
	},
	{
		{
			HXT5G02A_SYS_PALBG_SECAM_SWM, 
			HXT5G02A_SYS_PALBG_SECAM_ADJM, 
			HXT5G02A_SYS_PALBG_SECAM_DATM
		}
	},
	{
		{
			HXT5G02A_SYS_PALI_SWM, 
			HXT5G02A_SYS_PALI_ADJM, 
			HXT5G02A_SYS_PALI_DATM
		}
	},
	{
		{
			HXT5G02A_SYS_PALDK_SECAM_SWM, 
			HXT5G02A_SYS_PALDK_SECAM_ADJM, 
			HXT5G02A_SYS_PALDK_SECAM_DATM
		}
	},
	{
		{
			HXT5G02A_SYS_SECAMLL_SWM, 
			HXT5G02A_SYS_SECAMLL_ADJM, 
			HXT5G02A_SYS_SECAMLL_DATM
		}
	},
	{
		{
			HXT5G02A_SYS_FM_SWM, 
			HXT5G02A_SYS_FM_ADJM, 
			HXT5G02A_SYS_FM_DATM
		}
	}
};

TUNERCHANNEL	g_GlobalChannels[322]=
{
	{{0x91, 0x19, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "44.25"},	//	1
	{{0x95, 0x17, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "47.75"},	//	2
	{{0x95, 0x19, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "48.25"},	//	3
	{{0x97, 0x19, 0xE9, 0x66, 0x63, TCH_BW_TBD}, "49.75"},	//	4
	{{0x9C, 0x19, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "55.25"},	//	5
	{{0x8D, 0x17, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "55.75"},	//	6
	{{0x9F, 0x19, 0xE9, 0x66, 0x63, TCH_BW_TBD}, "57.75"},	//	7
	{{0x88, 0x17, 0xE6, 0x66, 0x63, TCH_BW_TBD}, "60.50"},	//	8
	{{0xA2, 0x19, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "61.25"},	//	9
	{{0xA3, 0x19, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "62.25"},	//	10
	{{0x85, 0x17, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "63.75"},	//	11
	{{0xA7, 0x19, 0xE9, 0x66, 0x63, TCH_BW_TBD}, "65.75"},	//	12
	{{0xA8, 0x19, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "67.25"},	//	13
	{{0xAA, 0x19, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "69.25"},	//	14
	{{0x81, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "76.25"},	//	15
	{{0x82, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "77.25"},	//	16
	{{0x88, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "83.25"},	//	17
	{{0x8A, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "85.25"},	//	18
	{{0x8F, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "90.25"},	//	19
	{{0x90, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "91.25"},	//	20
	{{0x96, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "97.25"},	//	21
	{{0x9C, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "103.25"},	//	22
	{{0x9E, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "105.25"},	//	23
	{{0xA2, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "109.25"},	//	24
	{{0xA5, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "112.25"},	//	25
	{{0xA8, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "115.25"},	//	26
	{{0xAC, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "119.25"},	//	27
	{{0xAD, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "120.25"},	//	28
	{{0xAE, 0x1A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "121.25"},	//	29
	{{0x83, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "126.25"},	//	30
	{{0x84, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "127.25"},	//	31
	{{0x85, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "128.25"},	//	32
	{{0x8A, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "133.25"},	//	33
	{{0x8D, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "136.25"},	//	34
	{{0x90, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "139.25"},	//	35
	{{0x91, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "140.25"},	//	36
	{{0x95, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "144.25"},	//	37
	{{0x96, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "145.25"},	//	38
	{{0x98, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "147.25"},	//	39
	{{0x9C, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "151.25"},	//	40
	{{0xA3, 0x1B, 0x0F, 0x65, 0x64, TCH_BW_TBD}, "152.25"},	//	41
	{{0x9F, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "154.25"},	//	42
	{{0xA2, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "157.25"},	//	43
	{{0xA5, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "160.25"},	//	44
	{{0xA6, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "161.25"},	//	45
	{{0xA8, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "163.25"},	//	46
	{{0xAA, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "165.25"},	//	47
	{{0xAD, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "168.25"},	//	48
	{{0xAE, 0x1B, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "169.25"},	//	49
	{{0x84, 0x1C, 0x07, 0x66, 0x64, TCH_BW_TBD}, "171.25"},	//	50
	{{0x84, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "175.25"},	//	51
	{{0x85, 0x1C, 0xE8, 0x66, 0x63, TCH_BW_TBD}, "176.00"},	//	52
	{{0x85, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "176.25"},	//	53
	{{0x86, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "177.25"},	//	54
	{{0x8A, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "181.25"},	//	55
	{{0x8B, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "182.25"},	//	56
	{{0x8C, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "183.25"},	//	57
	{{0x8D, 0x1C, 0xE8, 0x66, 0x63, TCH_BW_TBD}, "184.00"},	//	58
	{{0x8D, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "184.25"},	//	59
	{{0x90, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "187.25"},	//	60
	{{0x92, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "189.25"},	//	61
	{{0x92, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "189.25"},	//	62
	{{0x95, 0x1C, 0xE8, 0x66, 0x63, TCH_BW_TBD}, "192.00"},	//	63
	{{0x95, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "192.25"},	//	64
	{{0x96, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "193.25"},	//	65
	{{0x99, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "196.25"},	//	66
	{{0x9C, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "199.25"},	//	67
	{{0x9D, 0x1C, 0xE8, 0x66, 0x63, TCH_BW_TBD}, "200.00"},	//	68
	{{0x9D, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "200.25"},	//	69
	{{0xA0, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "203.25"},	//	70
	{{0xA2, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "205.25"},	//	71
	{{0xA2, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "205.25"},	//	72
	{{0xA5, 0x1C, 0xE8, 0x66, 0x63, TCH_BW_TBD}, "208.00"},	//	73
	{{0xA5, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "208.25"},	//	74
	{{0xA7, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "210.25"},	//	75
	{{0xA8, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "211.25"},	//	76
	{{0xAD, 0x1C, 0xE8, 0x66, 0x63, TCH_BW_TBD}, "216.00"},	//	77
	{{0xAD, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "216.25"},	//	78
	{{0xAE, 0x1C, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "217.25"},	//	79
	{{0x84, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "223.25"},	//	80
	{{0x85, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "224.25"},	//	81
	{{0x00, 0x00, 0x00, 0x00, 0x00, TCH_BW_TBD}, "00.00"},	//	82--ununsed
	{{0x00, 0x00, 0x00, 0x00, 0x00, TCH_BW_TBD}, "00.00"},	//	83--ununsed
	{{0x00, 0x00, 0x00, 0x00, 0x00, TCH_BW_TBD}, "00.00"},	//	84--ununsed
	{{0x00, 0x00, 0x00, 0x00, 0x00, TCH_BW_TBD}, "00.00"},	//	85--ununsed
	{{0x8A, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "229.25"},	//	86
	{{0x00, 0x00, 0x00, 0x00, 0x00, TCH_BW_TBD}, "00.00"},	//	87--ununsed
	{{0x8C, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "231.25"},	//	88
	{{0x8D, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "232.25"},	//	89
	{{0x90, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "235.25"},	//	90
	{{0x92, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "237.25"},	//	91
	{{0x93, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "238.25"},	//	92
	{{0x95, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "240.25"},	//	93
	{{0x96, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "241.25"},	//	94
	{{0x98, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "243.25"},	//	95
	{{0x9A, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "245.25"},	//	96
	{{0xA0, 0x1D, 0x07, 0x66, 0x64, TCH_BW_TBD}, "247.25"},	//	97
	{{0x9D, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "248.25"},	//	98
	{{0x9E, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "249.25"},	//	99
	{{0xA1, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "252.25"},	//	100
	{{0xA2, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "253.25"},	//	101
	{{0xA5, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "256.25"},	//	102
	{{0xA8, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "259.25"},	//	103
	{{0xAD, 0x1D, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "264.25"},	//	104
	{{0x82, 0x1E, 0x07, 0x66, 0x64, TCH_BW_TBD}, "265.25"},	//	105
	{{0xAD, 0x1D, 0xCF, 0x65, 0x63, TCH_BW_TBD}, "266.25"},	//	106
	{{0x84, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "271.25"},	//	107
	{{0x85, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "272.25"},	//	108
	{{0x86, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "273.25"},	//	109
	{{0x0A, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "277.25"},	//	110
	{{0x0D, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "280.25"},	//	111
	{{0x10, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "283.25"},	//	112
	{{0x14, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "287.25"},	//	113
	{{0x15, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "288.25"},	//	114
	{{0x16, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "289.25"},	//	115
	{{0x16, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "289.25"},	//	116
	{{0x1B, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "294.25"},	//	117
	{{0x1C, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "295.25"},	//	118
	{{0x1D, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "296.25"},	//	119
	{{0x22, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "301.25"},	//	120
	{{0x24, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "303.25"},	//	121
	{{0x25, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "304.25"},	//	122
	{{0x28, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "307.25"},	//	123
	{{0x2C, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "311.25"},	//	124
	{{0x2D, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "312.25"},	//	125
	{{0x2E, 0x1E, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "313.25"},	//	126
	{{0x04, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "319.25"},	//	127
	{{0x05, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "320.25"},	//	128
	{{0x0A, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "325.25"},	//	129
	{{0x0C, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "327.25"},	//	130
	{{0x0D, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "328.25"},	//	131
	{{0x10, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "331.25"},	//	132
	{{0x14, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "335.25"},	//	133
	{{0x15, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "336.25"},	//	134
	{{0x16, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "337.25"},	//	135
	{{0x1C, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "343.25"},	//	136
	{{0x1D, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "344.25"},	//	137
	{{0x22, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "349.25"},	//	138
	{{0x24, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "351.25"},	//	139
	{{0x25, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "352.25"},	//	140
	{{0x28, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "355.25"},	//	141
	{{0x2C, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "359.25"},	//	142
	{{0x2D, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "360.25"},	//	143
	{{0x2E, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "361.25"},	//	144
	{{0x2E, 0x1F, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "361.25"},	//	145
	{{0x0A, 0x20, 0x0F, 0x65, 0x64, TCH_BW_TBD}, "367.25"},	//	146
	{{0x04, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "367.25"},	//	147
	{{0x0B, 0x20, 0x0F, 0x65, 0x64, TCH_BW_TBD}, "368.25"},	//	148
	{{0x0C, 0x20, 0xEF, 0x65, 0x63, TCH_BW_TBD}, "373.25"},	//	149
	{{0x0C, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "375.25"},	//	150
	{{0x0D, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "376.25"},	//	151
	{{0x10, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "379.25"},	//	152
	{{0x14, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "383.25"},	//	153
	{{0x15, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "384.25"},	//	154
	{{0x16, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "385.25"},	//	155
	{{0x1C, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "391.25"},	//	156
	{{0x1D, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "392.25"},	//	157
	{{0x22, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "397.25"},	//	158
	{{0x24, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "399.25"},	//	159
	{{0x25, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "400.25"},	//	160
	{{0x28, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "403.25"},	//	161
	{{0x2C, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "407.25"},	//	162
	{{0x2D, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "408.25"},	//	163
	{{0x2E, 0x20, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "409.25"},	//	164
	{{0x0A, 0x21, 0x0F, 0x65, 0x64, TCH_BW_TBD}, "415.25"},	//	165
	{{0x04, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "415.25"},	//	166
	{{0x05, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "416.25"},	//	167
	{{0x0A, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "421.25"},	//	168
	{{0x0C, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "423.25"},	//	169
	{{0x0D, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "424.25"},	//	170
	{{0x10, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "427.25"},	//	171
	{{0x14, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "431.25"},	//	172
	{{0x15, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "432.25"},	//	173
	{{0x16, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "433.25"},	//	174
	{{0x1C, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "439.25"},	//	175
	{{0x1C, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "439.25"},	//	176
	{{0x1D, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "440.25"},	//	177
	{{0x22, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "445.25"},	//	178
	{{0x24, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "447.25"},	//	179
	{{0x25, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "448.25"},	//	180
	{{0x28, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "451.25"},	//	181
	{{0x2C, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "455.25"},	//	182
	{{0x2D, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "456.25"},	//	183
	{{0x2E, 0x21, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "457.25"},	//	184
	{{0x04, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "463.25"},	//	185
	{{0x04, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "463.25"},	//	186
	{{0x05, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "464.25"},	//	187
	{{0x0A, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "469.25"},	//	188
	{{0x0C, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "471.25"},	//	189
	{{0x10, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "475.25"},	//	190
	{{0x12, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "477.25"},	//	191
	{{0x14, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "479.25"},	//	192
	{{0x16, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "481.25"},	//	193
	{{0x18, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "483.25"},	//	194
	{{0x1C, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "487.25"},	//	195
	{{0x1E, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "489.25"},	//	196
	{{0x22, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "493.25"},	//	197
	{{0x24, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "495.25"},	//	198
	{{0x24, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "495.25"},	//	199
	{{0x28, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "499.25"},	//	200
	{{0x2A, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "501.25"},	//	201
	{{0x2C, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "503.25"},	//	202
	{{0x2E, 0x22, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "505.25"},	//	203
	{{0x00, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "507.25"},	//	204
	{{0x04, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "511.25"},	//	205
	{{0x06, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "513.25"},	//	206
	{{0x0A, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "517.25"},	//	207
	{{0x0C, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "519.25"},	//	208
	{{0x10, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "523.25"},	//	209
	{{0x12, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "525.25"},	//	210
	{{0x14, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "527.25"},	//	211
	{{0x16, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "529.25"},	//	212
	{{0x18, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "531.25"},	//	213
	{{0x1A, 0x23, 0xCF, 0x65, 0x63, TCH_BW_TBD}, "533.25"},	//	214
	{{0x1E, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "537.25"},	//	215
	{{0x22, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "541.25"},	//	216
	{{0x24, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "543.25"},	//	217
	{{0x28, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "547.25"},	//	218
	{{0x2A, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "549.25"},	//	219
	{{0x2C, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "551.25"},	//	220
	{{0x2E, 0x23, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "553.25"},	//	221
	{{0x00, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "555.25"},	//	222
	{{0x04, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "559.25"},	//	223
	{{0x06, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "561.25"},	//	224
	{{0x0A, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "565.25"},	//	225
	{{0x0C, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "567.25"},	//	226
	{{0x18, 0x24, 0x27, 0x66, 0x64, TCH_BW_TBD}, "571.25"},	//	227
	{{0x12, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "573.25"},	//	228
	{{0x14, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "575.25"},	//	229
	{{0x16, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "577.25"},	//	230
	{{0x18, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "579.25"},	//	231
	{{0x1C, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "583.25"},	//	232
	{{0x1E, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "585.25"},	//	233
	{{0x22, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "589.25"},	//	234
	{{0x24, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "591.25"},	//	235
	{{0x28, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "595.25"},	//	236
	{{0x2A, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "597.25"},	//	237
	{{0x2C, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "599.25"},	//	238
	{{0x2E, 0x24, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "601.25"},	//	239
	{{0x00, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "603.25"},	//	230
	{{0x04, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "607.25"},	//	241
	{{0x06, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "609.25"},	//	242
	{{0x0A, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "613.25"},	//	243
	{{0x0C, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "615.25"},	//	244
	{{0x10, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "619.25"},	//	245
	{{0x12, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "621.25"},	//	246
	{{0x14, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "623.25"},	//	247
	{{0x16, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "625.25"},	//	248
	{{0x18, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "627.25"},	//	249
	{{0x1C, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "631.25"},	//	250
	{{0x1E, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "633.25"},	//	251
	{{0x22, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "637.25"},	//	252
	{{0x24, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "639.25"},	//	253
	{{0x28, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "643.25"},	//	254
	{{0x2A, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "645.25"},	//	255
	{{0x2C, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "647.25"},	//	256
	{{0x2E, 0x25, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "649.25"},	//	257
	{{0x00, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "651.25"},	//	258
	{{0x04, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "655.25"},	//	259
	{{0x06, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "657.25"},	//	260
	{{0x0A, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "661.25"},	//	261
	{{0x0C, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "663.25"},	//	262
	{{0x10, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "667.25"},	//	263
	{{0x12, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "669.25"},	//	264
	{{0x14, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "671.25"},	//	265
	{{0x16, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "673.25"},	//	266
	{{0x18, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "675.25"},	//	267
	{{0x1C, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "679.25"},	//	268
	{{0x1E, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "681.25"},	//	269
	{{0x22, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "685.25"},	//	270
	{{0x24, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "687.25"},	//	271
	{{0x28, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "691.25"},	//	272
	{{0x2A, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "693.25"},	//	273
	{{0x2C, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "695.25"},	//	274
	{{0x2E, 0x26, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "697.25"},	//	275
	{{0x00, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "699.25"},	//	276
	{{0x04, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "703.25"},	//	277
	{{0x06, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "705.25"},	//	278
	{{0x0A, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "709.25"},	//	279
	{{0x0C, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "711.25"},	//	280
	{{0x10, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "715.25"},	//	281
	{{0x12, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "717.25"},	//	282
	{{0x14, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "719.25"},	//	283
	{{0x16, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "721.25"},	//	284
	{{0x18, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "723.25"},	//	285
	{{0x1C, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "727.25"},	//	286
	{{0x1E, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "729.25"},	//	287
	{{0x22, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "733.25"},	//	288
	{{0x24, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "735.25"},	//	289
	{{0x28, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "739.25"},	//	290
	{{0x2A, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "741.25"},	//	291
	{{0x2C, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "743.25"},	//	292
	{{0x2E, 0x27, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "745.25"},	//	293
	{{0x00, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "747.25"},	//	294
	{{0x04, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "751.25"},	//	295
	{{0x06, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "753.25"},	//	296
	{{0x0A, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "757.25"},	//	297
	{{0x0C, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "759.25"},	//	298
	{{0x16, 0x28, 0x0F, 0x65, 0x64, TCH_BW_TBD}, "764.25"},	//	299
	{{0x12, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "765.25"},	//	300
	{{0x14, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "767.25"},	//	301
	{{0x16, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "769.25"},	//	302
	{{0x18, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "771.25"},	//	303
	{{0x1C, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "775.25"},	//	304
	{{0x1E, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "777.25"},	//	305
	{{0x22, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "781.25"},	//	306
	{{0x24, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "783.25"},	//	307
	{{0x28, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "787.25"},	//	308
	{{0x29, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "788.25"},	//	309
	{{0x2C, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "791.25"},	//	310
	{{0x2E, 0x28, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "793.25"},	//	311
	{{0x00, 0x29, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "795.25"},	//	312
	{{0x04, 0x29, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "799.25"},	//	313
	{{0x06, 0x29, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "801.25"},	//	314
	{{0x0C, 0x29, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "807.25"},	//	315
	{{0x14, 0x29, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "815.25"},	//	316
	{{0x1C, 0x29, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "823.25"},	//	317
	{{0x24, 0x29, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "831.25"},	//	318
	{{0x2C, 0x29, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "839.25"},	//	319
	{{0x04, 0x2A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "847.25"},	//	320
	{{0x0C, 0x2A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "855.25"},	//	321
	{{0x14, 0x2A, 0xE7, 0x66, 0x63, TCH_BW_TBD}, "863.25"}	//	322
};

static int	g_pnUSAirChannels[CHNUM_US_AIR]=
{
	  1,   5,   9,  13,  16,  17,  51,  55,  60,  65, 
	 67,  71,  76, 189, 191, 194, 196, 199, 201, 204, 
	206, 208, 210, 213, 215, 217, 219, 222, 224, 226, 
	228, 231, 233, 235, 237, 240, 242, 244, 246, 249,
	251, 253, 255, 258, 260, 262, 264, 267, 269, 271,
	 -1, 276, 278, 280, 282, 285, 287, 289, 291, 294,
	296, 298, 300, 303, 305, 307, 309, 312, 314
	
};

static int	g_pnUSCATVChannels[CHNUM_US_CATV]=
{
	 -1,   5,   9,  13,  16,  17,  51,  55,  60,  65, 
	 67,  71,  76,  29,  31,  33,  35,  38,  40,  43,
	 46,  49,  79,  80,  86,  90,  94,  97, 101, 103,
	105, 107, 110, 112, 115, 118, 120, 123, 126, 127,
	129, 132, 135, 136, 138, 141, 144, 146, 149, 152,
	155, 156, 158, 161, 164, 165, 168, 171, 174, 175,
	178, 181, 184, 185, 188, 190, 193, 195, 197, 200,
	203, 205, 207, 209, 212, 214, 216, 218, 221, 223,
	225, 227, 230, 232, 234, 236, 239, 241, 243, 245,
	248, 250, 252, 254,  20,  21,  22,  24,  26, 257,
	259, 261, 263, 266, 268, 270, 272, 275, 277, 279,
	281, 284, 286, 288, 290, 293, 295, 297, 299, 302,
	304, 306, 308, 311, 313
};

static int	g_pnJapanAirChannels[CHNUM_JPN_AIR]=
{
	 20,  21,  22,  50,  54,  57,  62,  65,  67,  71, 
	 76,  79, 189, 191, 194, 196, 199, 201, 204, 206,
	208, 210, 213, 215, 217, 219, 222, 224, 226, 228,
	231, 233, 235, 237, 240, 242, 244, 246, 249, 251, 
	253, 255, 258, 260, 262, 264, 267, 269, 271, 273,
	276, 278, 280, 282, 285, 287, 289, 291, 294, 296, 
	298, 300
};

static int	g_pnJapanCATVChannels[CHNUM_JPN_CATV]=
{
	 -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1, 
	 -1,  -1,  24,  26,  29,  31,  33,  35,  38,  40,  
	 43,  47,  80,  88,  91,  95,  99, 101, 103, 105, 
	107, 110, 112, 116, 118, 120, 123, 126, 127, 129,
	132, 135, 136, 138, 141, 145, 147, 149, 152, 155, 
	156, 158, 161, 164, 165, 168, 171, 174, 176, 178, 
	181, 184, 186
};

//
//===========================================================================

//===========================================================================
//
static ctl_table HXT5G02A_dir_table_template[] = 
{
	{
		HXT5G02A_SYSCTL_READ, 
		"read", 
		NULL, 
		0, 
		0444, 
		NULL, 
		&i2c_proc_real,
		&i2c_sysctl_real, 
		NULL, 
		&HXT5G02AProcRead
	},
	{
		HXT5G02A_SYSCTL_WRITE, 
		"write", 
		NULL, 
		0, 
		0644, 
		NULL, 
		&i2c_proc_real,
		&i2c_sysctl_real, 
		NULL, 
		&HXT5G02AProcWrite
	},
	{0}
};

static ctl_table HXTTuner_dir_table_template[] = 
{
	{
		HXTTUNER_SYSCTL_READ, 
		"read", 
		NULL, 
		0, 
		0444, 
		NULL, 
		&i2c_proc_real,
		&i2c_sysctl_real, 
		NULL, 
		&HXTTunerProcRead
	},
	{
		HXTTUNER_SYSCTL_WRITE, 
		"write", 
		NULL, 
		0, 
		0644, 
		NULL, 
		&i2c_proc_real,
		&i2c_sysctl_real, 
		NULL, 
		&HXTTunerProcWrite
	},
	{0}
};

PHXT5G02ADATA 	g_pHXT5G02AData;
PHXTTUNERDATA	g_pHXTTunerData;
static int 		HXT5G02A_id = 0;
static int 		HXTTuner_id = 0;

static struct i2c_driver HXT5G02ADriver=
{
	.owner			= THIS_MODULE,
	.name			= "HXT5G02A system driver",		//	max. 40 characters limited
	.id				= I2C_DRIVERID_HXT5G02A,			//	usually from i2c-id.h
	.flags			= I2C_DF_NOTIFY,				//	always set this value
 	.attach_adapter	= HXT5G02AAttachAdapter,		
	.detach_client	= HXT5G02ADetachClient,
};

static struct i2c_driver HXTTunerDriver=
{
	.owner			= THIS_MODULE,
	.name			= "HXT5G02A Tuner driver",		//	max. 40 characters limited
	.id				= I2C_DRIVERID_HXTTUNER,		//	usually from i2c-id.h
	.flags			= I2C_DF_NOTIFY,				//	always set this value
 	.attach_adapter	= HXTTunerAttachAdapter,		
	.detach_client	= HXTTunerDetachClient,
};

//
//===========================================================================

//===========================================================================
//
void HXT5G02AProcRead(struct i2c_client *client, int operation,
	int ctl_name, int *nrels_mag, long *results)
{
	PHXT5G02ADATA 	data = client->data;
	int				i;
	
	if (operation == SENSORS_PROC_REAL_INFO)
		*nrels_mag = 0;
	else if (operation == SENSORS_PROC_REAL_READ) 
	{
		HXT5G02AUpdateClient(client);
		results[0]=data->wReg;
		for(i=0; i<3; i++)
			results[i+1] = data->byRead[i];
		*nrels_mag = 4;
	}  
}

void HXT5G02AProcWrite(struct i2c_client *client, int operation,
		    int ctl_name, int *nrels_mag, long *results)
{
	PHXT5G02ADATA	data = client->data;
	int				i;
	
	if (operation == SENSORS_PROC_REAL_INFO)
		*nrels_mag = 0;
	else if (operation == SENSORS_PROC_REAL_READ) 
	{
		results[0]=data->wReg;
		for(i=0; i<3; i++)
			results[i+1] = data->byRead[i];
		*nrels_mag = 4;
	} 
	else if (operation == SENSORS_PROC_REAL_WRITE) 
	{
		if (*nrels_mag == 4) 
		{
			data->wReg = results[0];
			for(i=0; i<3; i++)
			data->byWrite[i] = results[i+1];
			i2c_smbus_write_i2c_block_data(client, (unsigned char)data->wReg, 3, 
				data->byWrite);
		}
		
	}
}
//---------------------------------------------------------------------------
static int HXT5G02AAttachAdapter(struct i2c_adapter *adapter)
{
	int rc;
	rc=i2c_detect(adapter, &addr_data, HXT5G02ADetect);
	if(rc<0)
		printk("<1>call i2c_detect() failed\n");
	return rc;
}

//	This function is called by i2c_detect 
int HXT5G02ADetect(struct i2c_adapter *adapter, int address, unsigned short flags, int kind)
{
	struct i2c_client	*new_client;
	PHXT5G02ADATA 		data;
	int 				err = 0, i;
	const char			*type_name=NULL, *client_name=NULL;

//	printk("<1>HXT5G02ADetect() be called with address=0x%02X\n", address);
	
	//	Make sure we aren't probing the ISA bus!! This is just a safety check
	//  at this moment; i2c_detect really won't call us.
#ifdef DEBUG
	if (i2c_is_isa_adapter(adapter)) 
	{
		printk("HXT5G02A.o: HXT5G02ADetect called for an ISA bus adapter?!?\n");
		return 0;
	}
#endif

	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE | 
		I2C_FUNC_SMBUS_BYTE_DATA | I2C_FUNC_SMBUS_I2C_BLOCK))
	{
		printk("<1>HXT5G02ADetect() : i2c_check_functionality() failed\n");
		goto ERROR0;
	}
	//	OK. For now, we presume we have a valid client. We now create the
	//  client structure, even though we cannot fill it completely yet.
	//  But it allows us to access HXT5G02A_{read,write}_value.
	if (!(new_client = kmalloc(sizeof(struct i2c_client) + sizeof(HXT5G02ADATA), GFP_KERNEL))) 
	{
		err = -ENOMEM;
		goto ERROR0;
	}
	//	Now, we would do the remaining detection. But the HXT5G02A is plainly
	//  impossible to detect! Stupid chip.
	// Determine the chip type
	if (kind <= 0) 
	{
		if (address == HXT5G02A_I2C_IOADDR)
		{
			kind = HXT5G02A;
			type_name = "HXT5G02A";
			client_name = "HXT5G02A chip";
		}
		else 
			goto ERROR1;
	}
	else if (kind == HXT5G02A) 
	{
		type_name = "HXT5G02A";
		client_name = "HXT5G02A chip";
	} 
	else 
	{
#ifdef DEBUG
		printk("HXT5G02A.o: Internal error: unknown kind (%d)?!?", kind);
#endif
		goto ERROR1;
	}

	// Fill in the remaining client fields and put it into the global list 
	g_pHXT5G02AData= data = (HXT5G02ADATA *) (new_client + 1);
	new_client->addr = address;
	new_client->data = data;
	new_client->adapter = adapter;
	new_client->driver = &HXT5G02ADriver;
	new_client->flags = 0;
	strcpy(new_client->name, client_name);

	new_client->id = HXT5G02A_id++;
	data->cValid = 0;
	init_MUTEX(&data->UpdateLock);

	// Tell the I2C layer a new client has arrived
	if ((err = i2c_attach_client(new_client)))
		goto ERROR3;

	// Register a new directory entry with module sensors
	
	if ((i = i2c_register_entry(new_client, type_name, HXT5G02A_dir_table_template)) < 0) 
	{
		err = i;
		goto ERROR4;
	}
	
	data->nSysCtlID = i;

	//Initialize the HXT5G02AD chip
    HXT5G02AInitClient(new_client);
	return 0;

//	OK, this is not exactly good programming practice, usually. But it is
//	very code-efficient in this case.

ERROR4:
	i2c_detach_client(new_client);
ERROR3:
ERROR1:
	kfree(new_client);
ERROR0:
	return err;
}


static int HXT5G02ADetachClient(struct i2c_client *client)
{
	int err;

	i2c_deregister_entry(((PHXT5G02ADATA) (client->data))-> nSysCtlID);
	if ((err = i2c_detach_client(client))) 
	{
		printk("HXT5G02A.o: Client deregistration failed, client not detached.\n");
		return err;
	}
	kfree(client);
	return 0;

}
// Called when we have found a new HXT5G02A.

static void HXT5G02AInitClient(struct i2c_client *client)
{
	PHXT5G02ADATA 	data = client->data;
	
	data->pClient=client;
//	int				i, j, k;
//	printk("<1>HXT5G02AInitClient() be called\n");

	data->DevFS = devfs_register (NULL, "tuner_sys", 
		DEVFS_FL_AUTO_DEVNUM, 0, 0,	//	auto major and minor number
		S_IFCHR | S_IWUSR | S_IRUSR, &HXT5G02AFOPS, NULL);
	data->nVSTD=TVT_VSTD_NTSC_M;
	data->nOpen=0;
	memcpy(data->byWrite, g_SystemData[data->nVSTD].SystemData, 
		sizeof(unsigned char)*HXT5G02A_BLK_NR);
	i2c_smbus_write_i2c_block_data(client, HXT5G02AREG_SYSTEM, HXT5G02A_BLK_NR, 
		data->byWrite);	
	printk("<1>HXT5G02A TV tuner system is mounted\n");
}

static void HXT5G02AUpdateClient(struct i2c_client *client)
{
	HXT5G02ADATA 	*data = client->data;
	int				rc;

	down(&data->UpdateLock);
	if ((jiffies - data->uLastUpdated > 5*HZ) ||
	    (jiffies < data->uLastUpdated) || !data->cValid) 
	{
#ifdef DEBUG
		printk("Starting HXT5G02A update\n");
#endif
		rc = i2c_smbus_read_i2c_block_data(client, HXT5G02AREG_SYSTEM, data->byRead); 
		if(rc!=HXT5G02A_BLK_NR)
			printk("<1>i2c_smbus_read_i2c_block_data() return %d\n", rc);
		data->uLastUpdated = jiffies;
		data->cValid = 1;
	}
	up(&data->UpdateLock);
}
//---------------------------------------------------------------------------
static int HXT5G02AOpen(struct inode *pinode, struct file *pfile)
{
	if(g_pHXT5G02AData->nOpen)
		return -ENODEV;
	g_pHXT5G02AData->nOpen++;
	return 0;
}

static int HXT5G02ARelese(struct inode *pinode, struct file *pfile)
{
	if(!g_pHXT5G02AData->nOpen)
		return -ENODEV;
	g_pHXT5G02AData->nOpen--;
	return 0;
}

static int HXT5G02AIOCTL(struct inode *pinode, struct file *pfile, 
	unsigned int cmd, unsigned long arg)
{

	struct i2c_client	*pClient=g_pHXT5G02AData->pClient;
	int					nVSTD, rc=0;
	unsigned int		uData;
	
	switch (cmd)
	{
		case TVTSYS_SET_VSTD:
			nVSTD=(int)arg;
			if(nVSTD>HXT5G02A_SYS_MAX_STD || nVSTD <0)
			{
				rc= -EINVAL;
				break;
			}
			g_pHXT5G02AData->nVSTD=nVSTD;
			if(nVSTD<TVT_VSTD_PAL)
				nVSTD -= TVT_VSTD_NTSC;
			else if(nVSTD<TVT_VSTD_SECAM)
				nVSTD -= TVT_VSTD_PAL;
			else if(nVSTD<TVT_FM)
				nVSTD -= TVT_VSTD_SECAM;
			else
				nVSTD -= TVT_FM;
			printk("<1>sys program : 0x%02X, 0x%02X, 0x%02X\n",
				g_SystemData[nVSTD].SystemData[0], 
				g_SystemData[nVSTD].SystemData[1], 
				g_SystemData[nVSTD].SystemData[2]);
			i2c_smbus_write_i2c_block_data(pClient, HXT5G02AREG_SYSTEM, HXT5G02A_BLK_NR, 
				g_SystemData[nVSTD].SystemData);
			break;
		case TVTSYS_GET_VSTD:
			put_user(g_pHXT5G02AData->nVSTD, (int*)arg);
			break;
		case TVTSYS_SET_AFT:
			printk("<1>A-Ha~, TVTSYS_SET_AFT does not be implemented\n");
			break;
		case TVTSYS_GET_AFT:
			uData=i2c_smbus_read_byte_data(pClient, HXT5G02AREG_AFT);
			uData &= 0x000000FF;
			put_user(uData, (unsigned int*)arg);
			break;
		default:
			rc = -EINVAL;
	}
	return rc;
}
//
//===========================================================================

//===========================================================================
//
void HXTTunerProcRead(struct i2c_client *client, int operation,
	int ctl_name, int *nrels_mag, long *results)
{
	PHXTTUNERDATA 	data = client->data;
	int				i;
	
//	printk("<1>HXT5G02AProcRead():i2c_client = %p\n", client);
	if (operation == SENSORS_PROC_REAL_INFO)
		*nrels_mag = 0;
	else if (operation == SENSORS_PROC_REAL_READ) 
	{
		HXTTunerUpdateClient(client);
		results[0]=data->wReg;
		for(i=0; i<HXTTUNER_BLK_NR; i++)
			results[i+1] = data->byRead[i];
		*nrels_mag = HXTTUNER_BLK_NR+1;
	}  
}

void HXTTunerProcWrite(struct i2c_client *client, int operation,
		    int ctl_name, int *nrels_mag, long *results)
{
	PHXTTUNERDATA	data = client->data;
	int				i;
	
//	printk("<1>HXT5G02AProcWrite() be called\n");
	if (operation == SENSORS_PROC_REAL_INFO)
		*nrels_mag = 0;
	else if (operation == SENSORS_PROC_REAL_READ) 
	{
		results[0]=data->wReg;
		for(i=0; i<HXTTUNER_BLK_NR; i++)
			results[i+1] = data->byRead[i];
		*nrels_mag = HXTTUNER_BLK_NR+1;
	} 
	else if (operation == SENSORS_PROC_REAL_WRITE) 
	{
		if (*nrels_mag == HXTTUNER_BLK_NR+1) 
		{
			data->wReg = results[0];
			for(i=0; i<HXTTUNER_BLK_NR; i++)
			data->byWrite[i] = results[i+1];
			i2c_smbus_write_i2c_block_data(client, 
				(unsigned char)data->wReg, HXTTUNER_BLK_NR, 
				data->byWrite);
		}
	}
}
//---------------------------------------------------------------------------
static int HXTTunerAttachAdapter(struct i2c_adapter *adapter)
{
	int rc;
	rc=i2c_detect(adapter, &addr_data, HXTTunerDetect);
	if(rc<0)
		printk("<1>call i2c_detect() failed\n");
	return rc;
}

//	This function is called by i2c_detect 
int HXTTunerDetect(struct i2c_adapter *adapter, int address, unsigned short flags, 
	int kind)
{
	struct i2c_client	*new_client;
	PHXTTUNERDATA 		data;
	int 				err = 0, i;
	const char			*type_name=NULL, *client_name=NULL;

//	printk("<1>HXTTunerDetect() be called with address=0x%02X\n", address);
	
	//	Make sure we aren't probing the ISA bus!! This is just a safety check
	//  at this moment; i2c_detect really won't call us.
#ifdef DEBUG
	if (i2c_is_isa_adapter(adapter)) 
	{
		printk("HXT5G02A.o: HXTTunerDetect called for an ISA bus adapter?!?\n");
		return 0;
	}
#endif

	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE | 
		I2C_FUNC_SMBUS_BYTE_DATA | I2C_FUNC_SMBUS_I2C_BLOCK))
	{
		printk("<1>HXTTunerDetect() : i2c_check_functionality() failed\n");
		goto ERROR0;
	}
	//	OK. For now, we presume we have a valid client. We now create the
	//  client structure, even though we cannot fill it completely yet.
	//  But it allows us to access HXT5G02A_{read,write}_value.
	if (!(new_client = kmalloc(sizeof(struct i2c_client)+
		 sizeof(HXTTUNERDATA), GFP_KERNEL))) 
	{
		err = -ENOMEM;
		goto ERROR0;
	}
	//	Now, we would do the remaining detection. But the HXT5G02A is plainly
	//  impossible to detect! Stupid chip.
	// Determine the chip type
	if (kind <= 0) 
	{
		if (address == HXTTUNER_I2C_IOADDR)
		{
			kind = HXTTUNER;
			type_name = "HXTTUNER";
			client_name = "HXT5G02A Tuner";
		}
		else 
			goto ERROR1;
	}
	else if (kind == HXTTUNER) 
	{
		type_name = "HXTTUNER";
		client_name = "HXT5G02A Tuner";
	} 
	else 
	{
#ifdef DEBUG
		printk("HXT5G02A.o: Internal error: unknown kind (%d)?!?", kind);
#endif
		goto ERROR1;
	}

	// Fill in the remaining client fields and put it into the global list 
	g_pHXTTunerData= data = (HXTTUNERDATA *) (new_client + 1);
	new_client->addr = address;
	new_client->data = data;
	new_client->adapter = adapter;
	new_client->driver = &HXTTunerDriver;
	new_client->flags = 0;
	strcpy(new_client->name, client_name);

	new_client->id = HXTTuner_id++;
	data->cValid = 0;
	init_MUTEX(&data->UpdateLock);

	// Tell the I2C layer a new client has arrived
	if ((err = i2c_attach_client(new_client)))
		goto ERROR3;

	// Register a new directory entry with module sensors
	if ((i = i2c_register_entry(new_client, type_name, HXTTuner_dir_table_template)) < 0) 
	{
		err = i;
		goto ERROR4;
	}
	data->nSysCtlID = i;

	//Initialize the HXT5G02AD chip
    HXTTunerInitClient(new_client);
	return 0;

//	OK, this is not exactly good programming practice, usually. But it is
//	very code-efficient in this case.

ERROR4:
	i2c_detach_client(new_client);
ERROR3:
ERROR1:
	kfree(new_client);
ERROR0:
	return err;
}


static int HXTTunerDetachClient(struct i2c_client *client)
{
	int err;

	i2c_deregister_entry(((PHXTTUNERDATA) (client->data))-> nSysCtlID);
	if ((err = i2c_detach_client(client))) 
	{
		printk("HXT5G02A.o: Client deregistration failed, client not detached.\n");
		return err;
	}
	kfree(client);
	return 0;

}
// Called when we have found a new HXT5G02A.

static void HXTTunerInitClient(struct i2c_client *client)
{
	PHXTTUNERDATA 	data = client->data;
	data->pClient=client;
//	printk("<1>HXT5G02AInitClient() be called\n");
	
	data->DevFS = devfs_register (NULL, "pl_tuner", 
		DEVFS_FL_AUTO_DEVNUM, 0, 0,	//	auto major and minor number
		S_IFCHR | S_IWUSR | S_IRUSR, &HXTTUNERFOPS, NULL);
	data->ChannelData.nCurrInput= TCH_INPUT_US_AIR;
	data->ChannelData.nArea		= TVTUNER_AREA_USA;
	data->ChannelData.nCATV		= TVTUNER_TV;
	data->ChannelData.nChNum	= CHNUM_US_AIR;
	data->ChannelData.pChannels	= g_pnUSAirChannels;
	data->nOpen=0;	
	printk("<1>HXT5G02A TV tuner unit is mounted\n");
}

static void HXTTunerUpdateClient(struct i2c_client *client)
{
	PHXTTUNERDATA 	data = client->data;
	int				rc;

	down(&data->UpdateLock);
	if ((jiffies - data->uLastUpdated > 5*HZ) ||
	    (jiffies < data->uLastUpdated) || !data->cValid) 
	{
#ifdef DEBUG
		printk("Starting HXTTUNER update\n");
#endif
		rc = i2c_smbus_read_i2c_block_data(client, HXTTUNREREG_TUNER, data->byRead); 
		if(rc!=HXTTUNER_BLK_NR)
			printk("<1>i2c_smbus_read_i2c_block_data() return %d\n", rc);
		data->uLastUpdated = jiffies;
		data->cValid = 1;
	}
	up(&data->UpdateLock);
}
//---------------------------------------------------------------------------

static void SetChannel(int input)
{
	switch(input)
	{
		default:
		case TCH_INPUT_US_AIR:
			g_pHXTTunerData->ChannelData.nChNum=CHNUM_US_AIR;
			g_pHXTTunerData->ChannelData.pChannels=g_pnUSAirChannels;
			break;
		case TCH_INPUT_US_CATV:
			g_pHXTTunerData->ChannelData.nChNum=CHNUM_US_CATV;
			g_pHXTTunerData->ChannelData.pChannels=g_pnUSCATVChannels;
			break;
		case TCH_INPUT_CCIR:
			g_pHXTTunerData->ChannelData.nChNum=0;
			g_pHXTTunerData->ChannelData.pChannels=NULL;
			break;
		case TCH_INPUT_CCIR_CATV:
			g_pHXTTunerData->ChannelData.nChNum=0;
			g_pHXTTunerData->ChannelData.pChannels=NULL;
			break;
		case TCH_INPUT_FRANCE:
			g_pHXTTunerData->ChannelData.nChNum=0;
			g_pHXTTunerData->ChannelData.pChannels=NULL;
			break;
		case TCH_INPUT_CHINA:
			g_pHXTTunerData->ChannelData.nChNum=0;
			g_pHXTTunerData->ChannelData.pChannels=NULL;
			break;
		case TCH_INPUT_CHINA_CATV:
			g_pHXTTunerData->ChannelData.nChNum=0;
			g_pHXTTunerData->ChannelData.pChannels=NULL;
			break;
		case TCH_INPUT_JPN:
			g_pHXTTunerData->ChannelData.nChNum=CHNUM_JPN_AIR;
			g_pHXTTunerData->ChannelData.pChannels=g_pnJapanAirChannels;
			break;
		case TCH_INPUT_JPN_CATV:
			g_pHXTTunerData->ChannelData.nChNum=CHNUM_JPN_CATV;
			g_pHXTTunerData->ChannelData.pChannels=g_pnJapanCATVChannels;
			break;
		case TCH_INPUT_GLOBAL:
		case TCH_INPUT_GLOBAL_CATV:
			g_pHXTTunerData->ChannelData.nChNum=322;
			g_pHXTTunerData->ChannelData.pChannels=NULL;
			break;
	}
}

static int HXTTunerOpen(struct inode *pinode, struct file *pfile)
{
	if(g_pHXTTunerData->nOpen)
		return -ENODEV;
	g_pHXTTunerData->nOpen++;
	return 0;
}

static int HXTTunerRelese(struct inode *pinode, struct file *pfile)
{
	if(!g_pHXTTunerData->nOpen)
		return -ENODEV;
	g_pHXTTunerData->nOpen--;
	return 0;
}

static int HXTTunerIOCTL(struct inode *pinode, struct file *pfile, 
	unsigned int cmd, unsigned long arg)
{

	struct i2c_client	*pClient=g_pHXTTunerData->pClient;
	int					rc=0, ch, gch, nVSTD=g_pHXT5G02AData->nVSTD;
		
	switch (cmd)
	{
		case TVTUNER_SET_CHANNEL:
			ch=arg-1;
			if(g_pHXTTunerData->ChannelData.nChNum>ch)
			{
				if(g_pHXTTunerData->ChannelData.nCurrInput == TCH_INPUT_GLOBAL ||
					g_pHXTTunerData->ChannelData.nCurrInput == TCH_INPUT_GLOBAL_CATV)
				{
					gch=ch;
				}
				else if(g_pHXTTunerData->ChannelData.pChannels[ch]>0)
				{
					gch=g_pHXTTunerData->ChannelData.pChannels[ch]-1;
				}
				else
				{
					rc=-EINVAL;
					break;
				}
				memcpy(g_pHXTTunerData->byWrite, 
					g_GlobalChannels[gch].ChannelData,
					sizeof(unsigned char)*HXTTUNER_BLK_NR);
				printk("<1> %s MHz\n", g_GlobalChannels[gch].szFreq);
				if(nVSTD<TVT_VSTD_PAL)
					g_pHXTTunerData->byWrite[5]=TCH_BW_NTSC;
				else if(nVSTD<TVT_VSTD_SECAM)
					g_pHXTTunerData->byWrite[5]=TCH_BW_PAL;
				else if(nVSTD<TVT_FM)
					g_pHXTTunerData->byWrite[5]=TCH_BW_SECAM;
				else
					g_pHXTTunerData->byWrite[5]=TCH_BW_NTSC;
				printk("<1>set ch: 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X, 0x%02X\n", 
					g_pHXTTunerData->byWrite[0],
					g_pHXTTunerData->byWrite[1],
					g_pHXTTunerData->byWrite[2],
					g_pHXTTunerData->byWrite[3],
					g_pHXTTunerData->byWrite[4],
					g_pHXTTunerData->byWrite[5]);
				i2c_smbus_write_i2c_block_data(pClient, HXTTUNREREG_TUNER, 
						HXTTUNER_BLK_NR, g_pHXTTunerData->byWrite);
			}
			else
				rc=-EINVAL;
			break;
		case TVTUNER_GET_CHNUM:
			put_user(g_pHXTTunerData->ChannelData.nChNum, (int*)arg);
			break;
		case TVTUNER_SET_TV_CATV:
			g_pHXTTunerData->ChannelData.nCATV=arg;
			g_pHXTTunerData->ChannelData.nCurrInput=
				(1<<g_pHXTTunerData->ChannelData.nArea) |
				(g_pHXTTunerData->ChannelData.nCATV<<(g_pHXTTunerData->ChannelData.nArea+1));
			SetChannel(g_pHXTTunerData->ChannelData.nCurrInput);
			break;
		case TVTUNER_GET_TV_CATV:
			put_user(g_pHXTTunerData->ChannelData.nCATV, (int*)arg);
			break;
		case TVTUNER_SET_AREA:
			g_pHXTTunerData->ChannelData.nArea=arg;
			g_pHXTTunerData->ChannelData.nCurrInput=
				(1<<g_pHXTTunerData->ChannelData.nArea) |
				(g_pHXTTunerData->ChannelData.nCATV<<(g_pHXTTunerData->ChannelData.nArea+1));
			SetChannel(g_pHXTTunerData->ChannelData.nCurrInput);
			break;
		case TVTUNER_GET_AREA:
			put_user(g_pHXTTunerData->ChannelData.nArea, (int*)arg);
			break;
	}
	return 0;
}
//
//===========================================================================

//===========================================================================
//
static int __init sm_HXT5G02A_init(void)
{
	int		rc=0;
	printk("<1>HXT5G02A.o version %s (%s)\n", LM_VERSION, LM_DATE);
	rc	=i2c_add_driver(&HXT5G02ADriver);
	rc +=i2c_add_driver(&HXTTunerDriver);
	return rc;
}

static void __exit sm_HXT5G02A_exit(void)
{
	i2c_del_driver(&HXTTunerDriver);
	i2c_del_driver(&HXT5G02ADriver);
}
//
//===========================================================================

MODULE_AUTHOR("Chen Chao Yi<joe-chen@prolific.com.tw>");
MODULE_DESCRIPTION("HXT5G02A driver");

module_init(sm_HXT5G02A_init);
module_exit(sm_HXT5G02A_exit);


